@@ -1,5 +1,5 @@
 <?php
-// $Id: pager_test.php,v 1.22 2006/06/07 11:48:13 quipo Exp $
+// $Id: pager_test.php,v 1.31 2008/03/26 21:54:10 quipo Exp $
 
 require_once 'simple_include.php';
 require_once 'pager_include.php';
@@ -12,7 +12,7 @@ class TestOfPager extends UnitTestCase {
     }
     function setUp() {
         $options = array(
-            'itemData' => array(1, 2, 3, 4, 5, 6, 7, 8, 9, 10),
+            'itemData' => range(1, 10),
             'perPage'  => 5,
         );
         $this->pager = Pager::factory($options);
@@ -92,26 +92,57 @@ class TestOfPager extends UnitTestCase {
         $selectBox .= '<option value="4">4 bugs</option>';
         $selectBox .= '<option value="5" selected="selected">5 bugs</option>';
         $selectBox .= '<option value="6">6 bugs</option>';
-        $selectBox .= '<option value="10">Show All</option>';
+        $selectBox .= '<option value="'.max($this->pager->_itemData).'">Show All</option>';
         $selectBox .= '</select>';
         $this->assertEqual($selectBox, $this->pager->getPerPageSelectBox(3, 6, 1, true, '%d bugs'));
     }
     function testSelectBoxWithShowAllWithExtraAttribs() {
+        $options = array(
+            'itemData' => range(1, 14),
+            'perPage'  => 5,
+        );
+        $this->pager = Pager::factory($options);
         $this->pager->_showAllText = 'Show All';
         $selectBox  = '<select name="'.$this->pager->_sessionVar.'" onmouseover="doSth">';
-        $selectBox .= '<option value="3">3 bugs</option>';
-        $selectBox .= '<option value="4">4 bugs</option>';
         $selectBox .= '<option value="5" selected="selected">5 bugs</option>';
-        $selectBox .= '<option value="6">6 bugs</option>';
-        $selectBox .= '<option value="10">Show All</option>';
+        $selectBox .= '<option value="10">10 bugs</option>';
+        $selectBox .= '<option value="'.max($this->pager->_itemData).'">Show All</option>';
         $selectBox .= '</select>';
-        $params = array('optionText' => '%d bugs', 'attributes' => 'onmouseover="doSth"');
-        $this->assertEqual($selectBox, $this->pager->getPerPageSelectBox(3, 6, 1, true, $params));
+        $params = array(
+            'optionText'    => '%d bugs',
+            'attributes'    => 'onmouseover="doSth"',
+            'checkMaxLimit' => true,
+        );
+        $this->assertEqual($selectBox, $this->pager->getPerPageSelectBox(5, 15, 5, true, $params));
+    }
+    function testSelectBoxWithShowAllAndFewValues() {
+        //test getPerPagerSelectBox() with $showAllData = true and $start > $totalItems
+        $options = array(
+            'itemData'    => range(1, 5),
+            'perPage'     => 5,
+            'showAllText' => 'Show All',
+        );
+        $this->pager = Pager::factory($options);
+        $selectBox  = '<select name="'.$this->pager->_sessionVar.'">';
+        $selectBox .= '<option value="'.max($this->pager->_itemData).'" selected="selected">Show All</option>';
+        $selectBox .= '</select>';
+        $this->assertEqual($selectBox, $this->pager->getPerPageSelectBox(25, 30, 1, true, array('checkMaxLimit' => true)));
     }
     function testSelectBoxInvalid() {
         $err = $this->pager->getPerPageSelectBox(5, 15, 5, false, '%s bugs');
         $this->assertEqual(ERROR_PAGER_INVALID_PLACEHOLDER, $err->getCode());
     }
+    function testSelectBoxEmpty() {
+        $options = array(
+            'itemData'   => array(),
+            'totalItems' => 0,
+        );
+        $this->pager = Pager::factory($options);
+        $selectBox  = '<select name="'.$this->pager->_sessionVar.'">';
+        $selectBox .= '<option />';
+        $selectBox .= '</select>';
+        $this->assertEqual($selectBox, $this->pager->getPerPageSelectBox(5, 15, 5, false, array('checkMaxLimit' => true)));
+    }
     function testAppendInvalid() {
         $options = array(
             'totalItems' => 10,
@@ -232,6 +263,7 @@ class TestOfPager extends UnitTestCase {
                 0 => 'apple',
                 1 => 'orange'
             ),
+            'no' => 'test',
         );
         $actual = $this->pager->_getLinksData();
         $this->assertEqual($expected, $this->pager->_getLinksData());
@@ -239,7 +271,7 @@ class TestOfPager extends UnitTestCase {
         if ($separator == '&') {
             $separator = '&amp;';
         }
-        $expected = '<a href="'.$_SERVER['PHP_SELF'].'?arr%5B0%5D=apple'.$separator.'arr%5B1%5D=orange'.$separator.'pageID=2" title=""></a>';
+        $expected = '<a href="'.$_SERVER['PHP_SELF'].'?arr%5B0%5D=apple'.$separator.'arr%5B1%5D=orange'.$separator.'no=test'.$separator.'pageID=2" title=""></a>';
         $actual = $this->pager->_renderLink('', '');
         $this->assertEqual($expected, $actual);
     }
@@ -273,6 +305,19 @@ class TestOfPager extends UnitTestCase {
         $actual = $this->pager->_renderLink('', '');
         $this->assertEqual($expected, $actual);
     }
+    function testOnClick() {
+        $options = array(
+            'itemData'    => array(1, 2, 3, 4, 5, 6, 7, 8, 9, 10),
+            'perPage'     => 5,
+            'linkClass'   => 'testclass',
+            'onclick'  => 'doSomething(%d)',
+        );
+        $this->pager =& Pager::factory($options);
+
+        $expected = '<a href="'.$_SERVER['PHP_SELF'].'?pageID=2" class="testclass" onclick="doSomething(2)" title=""></a>';
+        $actual = $this->pager->_renderLink('', '');
+        $this->assertEqual($expected, $actual);
+    }
     function testImportQuery() {
         //add some fake url vars
         $_GET['arr'] = array(
@@ -431,6 +476,14 @@ class TestOfPager extends UnitTestCase {
         $expected = '<a href="' . $_SERVER['PHP_SELF'] . '?pageID=2" title="page 2">[LAST]</a>';
         $this->assertEqual($expected, $this->pager->_printLastPage());
     }
+    function testGetPageLinks() {
+        $links = $this->pager->_getPageLinks();
+        $this->assertTrue(false !== strpos($links, 'title="page 2"'));
+        $this->pager->setOptions(array('altPage' => '%d page'));
+        $this->pager->build();
+        $links = $this->pager->_getPageLinks();
+        $this->assertTrue(false !== strpos($links, 'title="2 page"'));
+    }
     function testGetBackLink() {
         $img = '&laquo;';
         $options = array(
@@ -443,6 +496,21 @@ class TestOfPager extends UnitTestCase {
         $expected = '<a href="' . $_SERVER['PHP_SELF'] . '?pageID=1" title="previous page">'.$img.'</a>&nbsp;';
         $this->assertEqual($expected, $this->pager->_getBackLink());
     }
+    function testGetBackLinkEmpty() {
+        $img = '[FIRST]';
+        $options = array(
+            'itemData' => array(1, 2, 3, 4, 5, 6, 7, 8, 9, 10),
+            'perPage'  => 5,
+        );
+        $this->pager = Pager::factory($options);
+        $expected = '';
+        $this->assertEqual($expected, $this->pager->_getBackLink());
+
+        $options['prevImgEmpty'] = $img;
+        $this->pager = Pager::factory($options);
+        $expected = $img.'&nbsp;';
+        $this->assertEqual($expected, $this->pager->_getBackLink());
+    }
     function testGetNexLink() {
         $img = '&raquo;';
         $options = array(
@@ -455,6 +523,22 @@ class TestOfPager extends UnitTestCase {
         $expected = '&nbsp;<a href="' . $_SERVER['PHP_SELF'] . '?pageID=2" title="next page">'.$img.'</a>&nbsp;';
         $this->assertEqual($expected, $this->pager->_getNextLink());
     }
+    function testGetNexLinkEmpty() {
+        $img = '[LAST]';
+        $options = array(
+            'itemData' => array(1, 2, 3, 4, 5, 6, 7, 8, 9, 10),
+            'perPage'  => 5,
+            'currentPage' => 2,
+        );
+        $this->pager = Pager::factory($options);
+        $expected = '';
+        $this->assertEqual($expected, $this->pager->_getNextLink());
+
+        $options['nextImgEmpty'] = $img;
+        $this->pager = Pager::factory($options);
+        $expected = '&nbsp;'.$img.'&nbsp;';
+        $this->assertEqual($expected, $this->pager->_getNextLink());
+    }
     function testHttpMethodAutoDetect() {
         $_POST['pageID'] = 3;
         $options = array(
@@ -556,5 +640,30 @@ class TestOfPager extends UnitTestCase {
         $expected = '<link rel="previous" href="'.$this->baseurl.'/myfile.1.php" title="previous page" />'."\n";
         $this->assertEqual($expected, $this->pager->_getPrevLinkTag());
     }
+    function testRelativeLinks() {
+        $fileName = array_pop(explode('/', $_SERVER['PHP_SELF']));
+        $options = array(
+            'mode'     => 'Sliding',
+            'path'     => '',
+            'fileName' => $fileName,
+            'itemData' => range('a', 'z'),
+            'spacesBeforeSeparator' => 0,
+            'spacesAfterSeparator'  => 0,
+        );
+        $this->pager = Pager::factory($options);
+        $expected = '<a href="' . $fileName . '?pageID=2" title="next page">&raquo;</a>';
+        $this->assertEqual($expected, $this->pager->_getNextLink());
+
+        $options = array(
+            'mode'     => 'Sliding',
+            'itemData' => range('a', 'z'),
+            'spacesBeforeSeparator' => 0,
+            'spacesAfterSeparator'  => 0,
+        );
+        $this->pager = Pager::factory($options);
+        $this->pager->build();
+        $expected = '<a href="' . $_SERVER['PHP_SELF'] . '?pageID=2" title="next page">&raquo;</a>';
+        $this->assertEqual($expected, $this->pager->_getNextLink());
+    }
 }
 ?>
\ No newline at end of file
